/*------------------------------------------------.
|   simple front end for printfax.		  |
| 						  |
|   qcc -O -o faxprint faxprint.c -lqmenu -lmin	  |
| 						  |
| (c) Jonathan Hudson 1998, freeely distributable |
`------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <qdos.h>
#include <qptr.h>

#include <things.h>
#include <menu.h>

void (*_SigStart)(void) = NULL;
char _prog_name[] = "faxprint";

long _stack = 12000;		/* MENU_REXT likes _lots_ of stack */

/*---------------------------------------------------.
| Find the current PROG_USE and DATA_USE directories |
`---------------------------------------------------*/

static void *xgetcwd (char *s)
{
    extern char *_sys_var;
    struct QLSTR **q;
    char *d = NULL;

    if (q = (struct QLSTR **) (_sys_var + 0xAC))
    {
	d = s;
	for (d = s; s != d + 80; s += 40, q++)
	{
	    strncpy (s, (*q)->qs_str, (*q)->qs_strlen);
	    *(s + (*q)->qs_strlen) = 0;
	}
    }
    return d;
}

/*------------------------------------------------------.
| Start a job. if flag == -1, then is independent (EX), |
| else its owned by us and synchronous (EW)	        |
`------------------------------------------------------*/

static int fork (short flag, char *prog, char *cmd)
{
    chanid_t chid;
    int jobid, rv = -1;

    if ((rv = io_open (prog, 0)) < 0)
    {
	char *fn, *dd, fnam[40], defdirs[80];
	if (xgetcwd (defdirs))
	{
	    short j;

	    for (dd = defdirs; rv < 0 && dd != defdirs + 80; dd += 40)
	    {
		fn = stpcpy (fnam, dd);
		stpcpy (fn, prog);
		rv = io_open (fnam, 0);
	    }
	}
    }

    if (rv > 0)
    {
	struct qdirect qdir;
	void *jobaddr;
	struct JOBHEADER *jh;	/* ha ha */
	long codsiz;
	short elen, nlen;

	chid = rv;
	if ((rv = fs_headr (chid, -1, &qdir, sizeof (qdir))) > 0)
	{
	    if (qdir.d_type == QF_PROG_TYPE)
	    {
		jobid = mt_cjob (qdir.d_length, qdir.d_datalen, NULL, 
				 (flag == -1) ? -1 : 0,
				 &jobaddr);
		if (jobid > 0)
		{
		    void *stk;

		    if ((rv = fs_load (chid, jobaddr, qdir.d_length)) > 0)
		    {
			io_close (chid);
			chid = 0;
			jh = (((struct JOBHEADER *) jobaddr) - 1);
			stk = (void *) jh->jb_regs.jb_A7;
			if (cmd && (nlen = strlen (cmd)))
			{
			    elen = (nlen + 1) & ~1;
			}
			else
			{
			    nlen = elen = 0;
			}

			((char *) stk) -= elen;
			(void *) jh->jb_regs.jb_A7 = stk;
			*((short *) stk)++ = 0;
			*((short *) stk)++ = nlen;
			while (nlen--)
			{
			    *((char *) stk)++ = *cmd++;
			}
			rv = mt_activ (jobid, 8, flag);
		    }
		    else
		    {
			mt_frjob (jobid, rv);
		    }
		}
	    }
	    else
	    {
		rv = -15;
	    }
	}
	if (chid > 0)
	{
	    io_close (chid);
	}
    }
    return rv;
}

/*------------------------------------------------------------------------.
| The main program module,  invoke as ex faxprint;'filename_fax'	  |
| 									  |
| Requests output type and output device, a sensible default is provided; |
| prt for print output, and a skeleton of win2_tmp_ for file types.	  |
| Translation to BASIC should be trivial.				  |
`------------------------------------------------------------------------*/

int main(int ac, char **av)
{
    int nitems, i, rv = ERR_BP;
    short nc;
    char *sp;
    
    static const char *pfdev[] =
    {
	"hp-pcl", "eps-24", "eps-9", "ps", "ps-lpr", "pbm", "pic",
	"pic-scaled", "gif", "gif-paged", "tiff", "tiff-packbits", "tiff-lzw"
    };

    /*----------------------------------------------------------.
    | Remove (any spurious) channels off stack to get file name |
    `----------------------------------------------------------*/

    sp = *(av + 1);
    nc = *((short *) (sp));
    sp += sizeof(short);
    sp +=  nc * sizeof(long);
    nc = *((short *) (sp));
    sp += sizeof(short);

    if(nc)
    {
	nitems = sizeof (pfdev) / sizeof (char *);
	
	/*-----------------------.
	| Ask for an output type |
	`-----------------------*/

	i = ListSelect ("Select", pfdev, nitems, 0, 0, 0, -1, -1,
			-1, -1, NULL);

	/*------------------------------------------------------.
	| If that's OK, get the device; give a sensible default |
	`------------------------------------------------------*/

	if(i > -1)
	{
	    int j;
	    char sbuf[40],buf[40];
	    struct QLSTR **q;
	    *buf = 0;

	    if(i == 0)
	    {
		if (q = (struct QLSTR ** ) (_sys_var + 0xb4))
		{
		    strncpy (sbuf, (*q)->qs_str, (*q)->qs_strlen);
		    *(sbuf + (*q)->qs_strlen) = 0;
		}
		else
		{
		    strcpy(sbuf, "prt");
		}
	    }
	    else
	    {
		char *s;
		if((s = getenv("TMP")))
		{
		    strcpy(sbuf, s);
		    free(s);
		}
		else
		{
		    strcpy(sbuf, "win2_tmp_");
		}
	    }
	    j = ReadString (_prog_name, "Output to", 
			    sbuf, 0x10000, buf, -1, -1, -1);

	    /*---------------------------------------------.
	    | If we got a device or file, run the printfax |
	    | command, asynchronously (EX)		   |
	    `---------------------------------------------*/

	    if(j == 0)
	    {
		char *p, cmd[128];
		int ns;

		p = stpcpy(cmd, "-t ");
		p = stpcpy(p, *(pfdev+i));
		p = stpcpy(p, " -o ");
		p = stpcpy(p, buf);
		*p++ = ' ';
		memcpy(p, sp, nc);
		p = p + nc;
		*p++ = '\0';
		fork(0, "printfax", cmd);
	    } 
	}
    }
    return rv;
}
int (*_Cstart) () = main;
